/******************************************************************
*
* File Name:    processArgs
*
* Purpose:      Supplies support for processing input and output
*               file names given as command line arguments
*
* Course ID:    DSAL.105.AC.01.04-B2/2003
* Student Name: Marius Joldos
* E-mail:       Marius.Joldos@cs.utcluj.ro
*
*******************************************************************
* Module History
*
* Date        Description
* -------     -------------------------
* 20080226    First submission of this module
******************************************************************/
#include <stdlib.h>
#include <libgen.h>
#include <stdarg.h>
#include "processArgs.h"
/*
  Usage
    prints ussage information
  Params:
	progName - name of program
  Returns:
	-
  Changes:
	-
*/
void Usage(const char *progName)
{
    // change this top reflect your programs usage
    // this example is for a graph problem
    fprintf( stderr, "%s demonstrates depth first search works\n", progName );
    fprintf( stderr, "Usage: %s [-i <inputFilename>] [-o <outputFilename>]\n", progName );
    fprintf( stderr, "\t <inputFilename> defaults to stdin\n" );
	fprintf( stderr, "\t <outputFilename> defaults to stdout\n" );
    fprintf( stderr, "Input structure:\n<number of vertices>\n" );
    fprintf( stderr, "Names (tag, alone on one line)\n" );
    fprintf( stderr, "<vertex number>,<name> vertices numbered from 0..n-1\n" );
    fprintf( stderr, "Arcs (tag, alone on one line)\n" );
    fprintf( stderr, "<tail vertex number> <head vertex number> note 1 space\n" );
    fprintf( stderr, "EOF\n" );
}
/*
  warn
  Params:
    fmtSpec - printf format specifier
	... - list of variables like for printf
  Returns:
	-
  Changes:
	-
*/
void warn(const char *fmtSpec, ...)
{
  va_list ap;
  va_start (ap, fmtSpec);
  vfprintf (stderr, fmtSpec, ap);
  va_end (ap);
  if (errno != 0) perror("\tReason: ");
}


/*
  processArgs
    Process the filenames given as command line arguments and, if possible,
    opens them;
    Arguments are given as:
        -ifilename or -i filename for input and
        -ofilename or -o filename for input
  Params:
    argc - argument count of main
    argv - argument vector of main
  Returns:
	-
  Changes:
	global file pointers fpIn for input and fpOut for output
*/
void processArgs(int argc, char *argv[])
{
    fpIn = fpOut = NULL;
 	if ( argc > 1 )
	{ /* we have args */
		while ( --argc )
		{ /* scan argument list backwards */
			if ( *argv[ argc ] == '-' )
			{ /* hyphen indicates a parameter */
				switch ( *( argv[ argc ] + 1 ) )
				{
					case 'i':	/* next arg is input file name. Open it */
					case 'I':
						if ( NULL != ( fpIn = fopen( argv[ argc + 1 ], "r" )))
                            break;
                        if ( argv[ argc + 1 ] )
                        {
   						    warn("Cannot open input %s\n", argv[ argc + 1 ]);
   						    break;
                        }
                        if (!fpIn)
                        {
                            if (NULL == (fpIn = fopen( argv[ argc ] + 2, "r" )))
                                warn("Cannot open input %s\n", argv[ argc ] + 2);
                        }
                        break;
					case 'o':
					case 'O': /* next arg is output file name. Open it */
						if ( NULL != ( fpOut = fopen( argv[ argc + 1 ], "r" )))
                            break;
                        if ( argv[ argc + 1 ] )
                        {
   						    warn("Cannot open output %s\n", argv[ argc + 1 ]);
   						    break;
                        }
                        if (!fpOut)
                        {
                            if (NULL == (fpOut = fopen( argv[ argc ] + 2, "r" )))
                                warn("Cannot open output %s\n", argv[ argc ] + 2);
                        }
                        break;
					default:
						/* change this accordingly */
						Usage(basename(argv[0]));
						exit( 1 );
				}
			} /* if current arg begins with '-' */
		} /* while have args */
	} /* if prog has args */
	if (!fpIn)
	{
        fprintf(stderr, "Input defaults to stdin\n");
        fpIn = stdin; /* set default */
	}
	if (!fpOut)
	{
        fprintf(stderr, "Output defaults to stdout\n");
        fpOut = stdout;  /* set default */
	}
}
